#include <iostream>
#include <deque>
#include <cassert>
#include <cstring>
#include <algorithm>
#include <set>
#include <vector>

using namespace std;

const int defaultNumCases = 100;
#ifdef DISABLE_BASH_COLOUR_CODING
	const string beginMessage, endMessage, beginError, endError, beginWarning, endWarning;
#else
	const string beginMessage = "\033[37m";
	const string endMessage = "\033[0m";
	const string beginError = "\033[31;1m";
	const string endError = endMessage;
	const string beginWarning = "\033[31;4m";
	const string endWarning = endMessage;
#endif

const int MAX_BOUNDARY_LENGTH = 1000; // from the problem statement
const int MAX_NUM_INSTRUCTIONS = 100; // also from the problem statement
const int ARRAY_SIZE = 3 * MAX_BOUNDARY_LENGTH;
const int startingPoint = ARRAY_SIZE / 2;

int randNumber(int a, int b) {
	assert(a <= b);
	int ret = (rand() % (b + 1 - a)) + a;
	assert(ret >= a && ret <= b);
	return ret;
}

char allDirections[][10] = {"x", "y", "z", "-x", "-y", "-z"};
int dx[] = {1, 0, -1, -1,  0,  1};
int dy[] = {0, 1,  1,  0, -1, -1};

struct instruction {
	int direction, length;
	instruction(int _d, int _l) : direction(_d), length(_l) {}
	friend ostream& operator<<(ostream& os, const instruction& inst) {
		assert(inst.direction >= 0 && inst.direction < 6);
		assert(inst.length >= 1 && inst.length < MAX_BOUNDARY_LENGTH);
		int sgn = (inst.direction < 3 ? 1 : -1);
		os << allDirections[inst.direction % 3] << " " << sgn * inst.length;
		return os;
	}
};

bool visited[ARRAY_SIZE][ARRAY_SIZE];

struct testcase {
	deque<instruction> instructions;
	void addInstruction(int d, int l) {
		instructions.push_back(instruction(d, l));
	}
	int omtrek() const {
		int som = 0;
		for (deque<instruction>::const_iterator it = instructions.begin(); it != instructions.end(); it++)
			som += it->length;
		return som;
	}
	bool verify(int casenum) const {
		if (instructions.size() == 0) {
			if (casenum == 0)
				cerr << beginError << "Warning: this mode has not yet been implemented." << endError << endl;
			return true;
		}
		if (instructions.size() < 3 || (int) instructions.size() > MAX_NUM_INSTRUCTIONS) {
			cerr << beginError << "Error in testcase generation: illegal number of instructions (" << instructions.size() << ")." << endError << endl;
			return false;
		}
		int som = 0;
		for (deque<instruction>::const_iterator it = instructions.begin(); it != instructions.end(); it++) {
			if (it->direction < 0 || it->direction >= 6) {
				cerr << beginError << "Error in testcase generation: illegal instruction (direction = " << it->direction << ")." << endError << endl;
				return false;
			}
			if (it->length < 1 || it->length >= MAX_BOUNDARY_LENGTH) {
				cerr << beginError << "Error in testcase generation: illegal instruction (length = " << it->length << ")." << endError << endl;
				return false;
			}
			som += it->length;
		}
		if (som < 3 || som >= MAX_BOUNDARY_LENGTH) {
			cerr << beginError << "Error in testcase generation: illegal boundary length (length = " << som << ")." << endError << endl;
			return false;
		}
		unsigned l = instructions.size();
		for (unsigned i = 0; i < l; i++) {
			unsigned j = (i + 1) % l;
			if (instructions[i].direction % 3 == instructions[j].direction % 3) {
				cerr << beginError << "Error in testcase generation: two consecutive edges in the same direction." << endError << endl;
				return false;
			}
		}
		memset(visited, 0, sizeof visited);
		int x = startingPoint;
		int y = startingPoint;
		for (deque<instruction>::const_iterator it = instructions.begin(); it != instructions.end(); it++) {
			for (int i = 0; i < it->length; i++) {
				x += dx[it->direction];
				y += dy[it->direction];
				if (visited[x][y]) {
					cerr << beginError << "Error in testcase generation: boundary touches itself." << endError << endl;
					return false;
				}
				visited[x][y] = true;
			}
		}
		if (x != startingPoint || y != startingPoint) {
			cerr << beginError << "Error in testcase generation: not a closed curve." << endError << endl;
			return false;
		}
		return true;
	}
	void keerom() {
		reverse(instructions.begin(), instructions.end());
		for (deque<instruction>::iterator it = instructions.begin(); it != instructions.end(); it++)
			it->direction = (it->direction + 3) % 6;
	}
	void schuifdoor(int aantal) {
		unsigned l = instructions.size();
		if (l == 0) return;
		aantal %= l;
		while (aantal < 0) aantal += l;
		if (aantal == 0) return;
		deque<instruction> oud = instructions;
		for (unsigned i = 0; i < l; i++)
			instructions[(i + aantal) % l] = oud[i];
	}
	void verticaalSpiegelen() {
		for (deque<instruction>::iterator it = instructions.begin(); it != instructions.end(); it++) {
			it->direction = (6 - it->direction) % 6;
		}
	}
	void append(int direction, int length = 1) {
		if (length == 0) return;
		if (instructions.empty() || instructions.back().direction % 3 != direction % 3) {
			instructions.push_back(instruction(direction, length));
		}
		else {
			int sgn = (direction < 3 ? 1 : -1);
			int curSgn = (instructions.back().direction < 3 ? 1 : -1);
			instructions.back().direction %= 3;
			instructions.back().length = curSgn * instructions.back().length + sgn * length;
			if (instructions.back().length == 0) instructions.pop_back();
			else if (instructions.back().length < 0) {
				instructions.back().direction += 3;
				instructions.back().length *= -1;
			}
		}
	}
	friend ostream& operator<<(ostream& os, const testcase& t) {
		if (t.instructions.size() == 0)
			os << "This mode has not yet been implemented." << endl;
		else {
			os << t.instructions.size() << endl;
			for (deque<instruction>::const_iterator it = t.instructions.begin(); it != t.instructions.end(); it++)
				os << *it << endl;
		}
		return os;
	}
};

struct punt {
	int x, y, helft;
	punt(int _x, int _y, int _h) : x(_x), y(_y), helft(_h) {}
	vector<punt> buren() const {
		vector<punt> r;
		r.push_back(punt(x, y, 1 - helft));
		r.push_back(punt(x + 2 * helft - 1, y, 1 - helft));
		r.push_back(punt(x, y + 2 * helft - 1, 1 - helft));
		return r;
	}
	bool operator<(const punt& other) const {
		if (y != other.y) return y < other.y;
		if (x != other.x) return x < other.x;
		return helft < other.helft;
	}
	bool operator==(const punt& other) const {
		return x == other.x && y == other.y && helft == other.helft;
	}
	bool operator!=(const punt& other) const {
		return !(*this == other);
	}
};

punt volgende(const punt& huidige, int direction) {
	punt ret(huidige);
	ret.x += dx[direction];
	ret.y += dy[direction];
	ret.helft = 0;
	return ret;
}

testcase vindBuitenRand(const set<punt>& punten) {
	testcase ret;
	int startX = punten.begin()->x;
	int startY = punten.begin()->y + punten.begin()->helft;
	punt cur(startX, startY, 0);
	punt startpunt(cur);
	int direction = (punten.begin()->helft == 0 ? 0 : 5);
	while (volgende(cur, direction) != startpunt) {
		punt v = volgende(cur, direction);
		int newDirection;
		bool success = false;
		for (int i = 0; i < 6 && !success; i++) {
			newDirection = (direction + 4 + i) % 6;
			int ddx[] = {0, -1, -1, -1,  0,  0};
			int ddy[] = {0,  0,  0, -1, -1, -1};
			int ddh[] = {0,  1,  0,  1,  0,  1};
			punt w(v.x + ddx[newDirection],
			       v.y + ddy[newDirection],
			       ddh[newDirection]);
			if (punten.find(w) != punten.end())
				success = true;
		}
		assert(success);
		assert(direction != (newDirection + 3) %6);
		ret.append(direction);
		cur = v;
		direction = newDirection;
	}
	ret.append(direction);
	if (ret.instructions.size() >= 2 && ret.instructions[0].direction % 3 == ret.instructions.back().direction % 3) {
		instruction inst = ret.instructions[0];
		ret.instructions.pop_front();
		ret.append(inst.direction, inst.length);
	}
	return ret;
}

int main() {
	srand(time(NULL));
	int numTriangles;
	cin >> numTriangles;
	set<punt> s;
	for (int i = 0; i < numTriangles; i++) {
		int x, y, h;
		cin >> x >> y >> h;
		s.insert(punt(x, y, h));
	}
	assert((int) s.size() == numTriangles);
	testcase t = vindBuitenRand(s);
	t.verticaalSpiegelen();
	int keerom = randNumber(0,1);
	if (keerom) {
		t.keerom();
	}
	int schuif = randNumber(1, t.instructions.size());
	t.schuifdoor(schuif);
	if (!t.verify(1)) {
		cerr << beginError << "Waarschuwing: de gegenereerde testcase is ongeldig!" << endError << endl;
	}
	cout << 1 << endl;
	cout << t;
	return 0;
}
