#include <iostream>
#include <cstdio>
#include <cstring>
#include <string>
#include <cstdlib>
#include <vector>

using namespace std;

const int MAX_NUM_TESTCASES = 1000;
const int MAX_BOUNDARY_LENGTH = 1000;

#ifdef DISABLE_BASH_COLOUR_CODING
	const string beginMessage, endMessage, beginError, endError, beginWarning, endWarning;
#else
	const string beginMessage = "\033[37m";
	const string endMessage = "\033[0m";
	const string beginError = "\033[31;1m";
	const string endError = endMessage;
	const string beginWarning = "\033[31;4m";
	const string endWarning = endMessage;
#endif


int linenum = 0;

int readNumber(int minVal, int maxVal) {
	string str;
	if (!getline(cin, str)) {
		cerr << beginError << "Error: file ended prematurely." << endError << endl;
		throw 100;
	}
	linenum++;
	if (str.find_first_of('\r') != string::npos) {
		cerr << beginError << "Error: carriage return character detected ('\\r')." << endError << endl;
		throw 102;
	}
	int val;
	sscanf(str.c_str(), "%d", &val);
	char tmp[30];
	sprintf(tmp, "%d", val);
	if (strcmp(str.c_str(), tmp)) {
		cerr << beginError << "Error on line " << linenum << ": failed to read integer (\"" << str << "\" != \"" << tmp << "\")." << endError << endl;
		throw 1;
	}
	if (val < minVal || val > maxVal) {
		cerr << beginError << "Error on line " << linenum << ": integer out of range (val = " << val << ", minVal = " << minVal << ", maxVal = " << maxVal << ")." << endError << endl;
		throw 2;
	}
	return val;
}

struct instruction {
	char dir;
	int num;
};

int dx[] = {1, 0, -1};
int dy[] = {0, 1,  1};

struct point {
	int x, y;
};

instruction readInstruction(int minNum, int maxNum) {
	string str;
	if (!getline(cin, str)) {
		cerr << beginError << "Error: file ended prematurely." << endError << endl;
		throw 100;
	}
	linenum++;
	if (str.find_first_of('\r') != string::npos) {
		cerr << beginError << "Error: carriage return character detected ('\\r')." << endError << endl;
		throw 102;
	}
	instruction inst;
	sscanf(str.c_str(), "%c %d", &inst.dir, &inst.num);
	char tmp[30];
	sprintf(tmp, "%c %d", inst.dir, inst.num);
	if (strcmp(str.c_str(), tmp)) {
		cerr << beginError << "Error on line " << linenum << ": failed to read instruction (\"" << str << "\" != \"" << tmp << "\")." << endError << endl;
		throw 1;
	}
	if (abs(inst.num) < minNum || abs(inst.num) > maxNum) {
		cerr << beginError << "Error on line " << linenum << ": integer out of range (num = " << inst.num << ", minNum = " << minNum << ", maxNum = " << maxNum << ")." << endError << endl;
		if (inst.num != 0) {
			cerr << beginMessage << "You probably exceeded the maximum allowed total boundary length (" << MAX_BOUNDARY_LENGTH << ")." << endMessage << endl;
		}
		throw 2;
	}
	if (inst.dir != 'x' && inst.dir != 'y' && inst.dir != 'z') {
		cerr << beginError << "Error on line " << linenum << ": invalid direction ('" << inst.dir << "')." << endError << endl;
		throw 3;
	}
	return inst;
}

int visited[3 * MAX_BOUNDARY_LENGTH][3 * MAX_BOUNDARY_LENGTH];

int sgn(int x) {
	if (x > 0) return 1;
	if (x < 0) return -1;
	return 0;
}

void solve() {
	int C = readNumber(3, 100);
	int remainingBoundary = MAX_BOUNDARY_LENGTH;
	int startingPoint = (3 * MAX_BOUNDARY_LENGTH) / 2;
	int x = startingPoint;
	int y = startingPoint;
	memset(visited, 0, sizeof visited);
	vector<instruction> allInstructions;
	for (int i = 0; i < C; i++) {
		instruction inst = readInstruction(1, remainingBoundary);
		remainingBoundary -= abs(inst.num);
		for (int j = 0; j < abs(inst.num); j++) {
			x += dx[inst.dir - 'x'] * sgn(inst.num);
			y += dy[inst.dir - 'x'] * sgn(inst.num);
			if (visited[x][y]) {
				cerr << beginError << "Error on line " << linenum << ": coordinates (" << x - startingPoint << ", " << y - startingPoint << ") were already visited before." << endError << endl;
				throw 4;
			}
			visited[x][y]++;
		}
		allInstructions.push_back(inst);
		// the following line used to be included for debugging purposes.
		// cerr << "  inst = {" << inst.dir << ", " << inst.num << "}; pos = (" << x - startingPoint << ", " << y - startingPoint << ")." << endl;
	}
	if (x != startingPoint || y != startingPoint) {
		cerr << beginError << "Error on line " << linenum << ": route does not end at starting point." << endError << endl;
		throw 5;
	}
	for (int i = 0; i < C; i++) {
		int j = (i + 1) % C;
		if (allInstructions[i].dir == allInstructions[j].dir) {
			int k = max(i, j);
			int correctedLineNum = linenum - C + k + 1;
			cerr << beginError << "Error on line " << correctedLineNum << ": two consecutive segments in the same direction." << endError << endl;
			throw 6;
		}
	}
}

int main() {
	cerr << beginMessage << "Note: in order to keep this program simple, I will quit on the first error." << endMessage << endl;
	try {
		int T = readNumber(1, MAX_NUM_TESTCASES);
		for (int i = 0; i < T; i++) {
			solve();
			cout << "Case #" << (i + 1) << ": OK." << endl;
		}
		if (cin.fail() || cin.eof()) {
			cerr << beginError << "Error: file ended prematurely (perhaps missing final newline)." << endError << endl;
			throw 100;
		}
		string str;
		if (getline(cin, str)) {
			cerr << beginError << "Error: detected additional whitespace after last test case." << endError << endl;
			throw 101;
		}
	}
	catch (int x) {
		cerr << beginMessage << "Terminating now..." << endMessage << endl;
		return x;
	}
	cerr << beginMessage << "Congratulations, no errors were found!" << endMessage << endl;
	return 0;
}
