// O(MAX_BOUNDARY_LENGTH * MAX_BOUNDARY_LENGTH) solution based on bookkeeping.
//
// For every horizontal row of triangles we keep track of the "vertical" lines
// intersecting this row (either in y or z direction). Then we walk through the
// row from left to right and we keep track of the vertical lines we encounter.
// After every vertical line we know whether we are outside or inside the
// jewel, so we know whether or not we have to count the encountered triangles
// towards the grand total.
// 
// This solution should be fast enough, though significantly slower than the
// solution based on the shoelace formula.

#include <iostream>
#include <cassert>
#include <cstdlib>
#include <cstring>

using namespace std;

const int MAX_NUM_CUTS = 100;
const int MAX_BOUNDARY_LENGTH = 1000;
const int ARRAY_SIZE = (MAX_BOUNDARY_LENGTH * 10) / 9;
const int startingPoint = ARRAY_SIZE / 2;

bool hasY[ARRAY_SIZE][ARRAY_SIZE];
bool hasZ[ARRAY_SIZE][ARRAY_SIZE];

int dx[] = {1, 0, -1};
int dy[] = {0, 1,  1};

int sgn(int x) {
	if (x > 0) return 1;
	if (x < 0) return -1;
	return 0;
}

int solve() {
	int C;
	cin >> C;
	assert(C >= 3 && C <= MAX_NUM_CUTS);
	memset(hasY, 0, sizeof hasY);
	memset(hasZ, 0, sizeof hasZ);
	int x = startingPoint;
	int y = startingPoint;
	for (int i = 0; i < C; i++) {
		char dir;
		int len;
		cin >> dir >> len;
		assert(dir == 'x' || dir == 'y' || dir == 'z');
		assert(len != 0 && abs(len) <= MAX_BOUNDARY_LENGTH);
		for (int j = 0; j < abs(len); j++) {
			int oldX = x;
			int oldY = y;
			x += sgn(len) * dx[dir - 'x'];
			y += sgn(len) * dy[dir - 'x'];
			if (dir == 'y') {
				if (len > 0) {
					hasY[x][y] = true;
				}
				else {
					hasY[oldX][oldY] = true;
				}
			}
			if (dir == 'z') {
				if (len > 0) {
					hasZ[x][y] = true;
				}
				else {
					hasZ[oldX][oldY] = true;
				}
			}
		}
	}
	int numTriangles = 0;
	for (int y = 0; y < ARRAY_SIZE; y++) {
		bool inside = false;
		for (int x = 0; x < ARRAY_SIZE; x++) {
			if (hasY[x][y]) inside = !inside;
			if (inside) numTriangles++;
			if (hasZ[x][y]) inside = !inside;
			if (inside) numTriangles++;
		}
	}
	return numTriangles;
}

int main() {
	int T;
	cin >> T;
	for (int i = 0; i < T; i++) {
		cout << solve() << endl;
	}
	return 0;
}
