// Compare script for Chess Tournament (BAPC preliminaries 2015)
// 
// Authors: Josse, Raymond.
// 
// One can define the following flags in order to be lenient towards various
// forms of slightly malformed output:
// 
//    IGNORE_WHITESPACE        ignore additional whitespace in the list of team
//                             members for the first team.
//    
//    IGNORE_FINAL_NEWLINE     don't reject the answer if the team forgot to
//                             print a newline after the final test case.
//    
//    IGNORE_ADDITIONAL_DATA   if the team prints additional data after the
//                             final test case, ignore it.
//    

#include <cctype>
#include <cstdio>
#include <cstdlib>
#include <cstring>
#include <fstream>
#include <iostream>
#include <set>
#include <sstream>
#include <string>
#include <vector>

using namespace std;

const int MAX_N     = 250;
const int MAX_TEAMS = 2 * MAX_N;
const int MAX_M     = 1000000;
const long long MOD = 100000000;

ifstream inputfile;
ifstream juryoutput;
ofstream commentsfile;

void protectedGetline(ifstream& in, const string &filename, string& s, char delim = '\n') {
	if (!getline(in, s, delim)) {
		cerr << "Error: input file (\"" << filename << "\") ended unexpectedly." << endl;
		commentsfile << "Error: input file (\"" << filename << "\") ended unexpectedly." << endl;
		exit(1);
	}
}

string removeWhitespace(const string& s, bool &changed) {
	string ret;
	unsigned l = s.size();
	for (unsigned i = 0; i < l; i++) {
		if (!isspace(s[i]))
			ret.push_back(s[i]);
		else if (!ret.empty() && ret[ret.size() - 1] != ' ')
			ret.push_back(' ');
	}
	if (!ret.empty() && ret[ret.size() - 1] == ' ') ret.resize(ret.size() - 1);
	changed = (ret != s);
	return ret;
}

int matchesPlayed[MAX_TEAMS][MAX_TEAMS];
int teamNum[MAX_TEAMS];

int main(int argc, char* argv[]) {
	// check for missing arguments:
	if (argc < 4) {
		cerr << "Error in invocation of " << argv[0] << ": missing argument";
		if (argc < 3) cerr << 's';
		cerr << "." << endl;
		cout << "Internal error." << endl;
		return 1;
	}
	// open inputfile:
	string inputfilename(argv[1]);
	inputfile.open(inputfilename.c_str());
	if (inputfile.fail()) {
		cerr << "Error in " << argv[0] << ": failed to open input file (\"" << inputfilename << "\")." << endl;
		cout << "Internal error." << endl;
		return 1;
	}
	// open juryoutput:
	string juryoutputfilename(argv[2]);
	juryoutput.open(juryoutputfilename.c_str());
	if (juryoutput.fail()) {
		cerr << "Error in " << argv[0] << ": failed to open jury output (\"" << juryoutputfilename << "\")." << endl;
		cout << "Internal error." << endl;
		return 1;
	}
	// open commentsfile:
	string commentsfilename(argv[3]);
	if (commentsfilename.empty() || commentsfilename[commentsfilename.size() - 1] != '/') {
		commentsfilename.push_back('/');
	}
	commentsfilename.append("judgemessage.txt");
	commentsfile.open(commentsfilename.c_str());
	if (commentsfile.fail()) {
		cerr << "Error in " << argv[0] << ": failed to open comments file (\"" << commentsfilename << "\")." << endl;
		cout << "Internal error." << endl;
		return 1;
	}
	// produce a warning if there are too many arguments:
	if (argc > 4) {
		commentsfile << "Warning: ignoring additional arguments (";
		for (int i = 4; i < argc; i++) {
			if (i > 4) cerr << ", ";
			commentsfile << '"' << argv[i] << '"';
		}
		commentsfile << ")." << endl;
	}
	// actual verification starts HERE:
	int T;
	string s;
	protectedGetline(inputfile, inputfilename, s);
	sscanf(s.c_str(), "%d", &T);
	int numWrongAnswers = 0;
	int whitespacecount = 0;
	for (int casenum = 0; casenum < T; casenum++) {
		// read input:
		int N, M;
		protectedGetline(inputfile, inputfilename, s);
		sscanf(s.c_str(), "%d %d", &N, &M);
		if (N < 1 || N > MAX_N || M < N*N || M > MAX_M) {
			commentsfile << "Error: input error occurred while reading input file (\"" << inputfilename << "\")." << endl;
			cout << "Internal error." << endl;
			return 1;
		}
		memset(matchesPlayed, 0, sizeof matchesPlayed);
		for (int i = 0; i < M; i++) {
			int A, B;
			protectedGetline(inputfile, inputfilename, s);
			sscanf(s.c_str(), "%d %d", &A, &B);
			if (A < 1 || A > 2 * N || B < 1 || B > 2 * N || A == B) {
				commentsfile << "Error: input error occurred while reading input file (\"" << inputfilename << "\")." << endl;
				cout << "Internal error." << endl;
				return 1;
			}
			A--;
			B--;
			matchesPlayed[A][B]++;
			matchesPlayed[B][A]++;
		}
		// read jury output:
		long long ans;
		string ansString;
		protectedGetline(juryoutput, juryoutputfilename, ansString);
		sscanf(ansString.c_str(), "%lld", &ans);
		char tmp[100];
		sprintf(tmp, "%lld", ans);
		if (ans < 0 || ans >= MOD || ansString != string(tmp)) {
			commentsfile << "Error: input error occurred while reading jury output file (\"" << juryoutputfilename << "\")." << endl;
			cout << "Internal error." << endl;
			return 1;
		}
		protectedGetline(juryoutput, juryoutputfilename, s); // ignore next line
		// read team output:
		string teamAnsString;
		bool whitespacefail = false;
		vector<long long> firstTeam;
		bool doPush = true;
		if (!getline(cin, teamAnsString)) {
			commentsfile << "Problem with team output: unexpected end of file (" << T - casenum << " test case" << (casenum == T - 1 ? "" : "s") << " remaining)." << endl;
			return 43;
		}
		if (!getline(cin, s)) {
			commentsfile << "Problem with team output: unexpected end of file (" << T - casenum << " test case" << (casenum == T - 1 ? "" : "s") << " remaining)." << endl;
			return 43;
		}
		commentsfile << "Case " << casenum + 1 << ": ";
		if (teamAnsString.empty()) {
			commentsfile << "empty line detected." << endl;
			goto wrongAnswer;
		}
		#ifdef IGNORE_WHITESPACE
			teamAnsString = removeWhitespace(teamAnsString, whitespacefail);
		#endif
		if (teamAnsString != ansString) {
			commentsfile << "number of solutions incorrect or possibly malformed (jury solution = " << ans << ", team solution";
			if (whitespacefail) {
				commentsfile << " after clearing up additional whitespace";
			}
			commentsfile << " = \"" << teamAnsString << "\")." << endl;
			goto wrongAnswer;
		}
		if (s.empty()) {
			commentsfile << "empty line detected." << endl;
			goto wrongAnswer;
		}
		if (s[0] == ' ') {
			#ifdef IGNORE_WHITESPACE
				whitespacefail = true;
			#else
				commentsfile << "line starts with a space." << endl;
				goto wrongAnswer;
			#endif
		}
		if (s.size() > 1 && s[s.size() - 1] == ' ') {
			#ifdef IGNORE_WHITESPACE
				whitespacefail = true;
			#else
				commentsfile << "line ends with a space." << endl;
				goto wrongAnswer;
			#endif
		}
		for (unsigned i = 0; i < s.size(); i++) {
			#ifdef IGNORE_WHITESPACE
				if (!isdigit(s[i]) && s[i] != ' ' && s[i] != '\t')
			#else
				if (!isdigit(s[i]) && s[i] != ' ')
			#endif
			{
				commentsfile << "illegal character (";
				if (isprint(s[i])) {
					commentsfile << "'" << s[i] << "'";
				}
				else if (s[i] == '\t') {
					commentsfile << "'\\t'";
				}
				else {
					char tmp[20];
					sprintf(tmp, "0x%02X", s[i]);
					commentsfile << tmp;
				}
				commentsfile << ") detected." << endl;
				goto wrongAnswer;
			}
			if (i > 0 && isspace(s[i]) && isspace(s[i - 1])) {
				#ifdef IGNORE_WHITESPACE
					whitespacefail = true;
				#else
					commentsfile << "two or more consecutive spaces detected." << endl;
					goto wrongAnswer;
				#endif
			}
			if (isdigit(s[i])) {
				if (doPush) {
					firstTeam.push_back(0);
					if (s[i] == '0' && i < s.size() - 1 && !isspace(s[i + 1])) {
						commentsfile << "leading zero detected (this is not allowed)." << endl;
						goto wrongAnswer;
					}
				}
				if (firstTeam.empty()) {
					commentsfile << "unexpected error. This should never happen; please contact Josse." << endl;
					cerr << "An unexpected error occurred." << endl;
					cout << "Internal error." << endl;
					return 1;
				}
				doPush = false;
				firstTeam.back() *= 10;
				firstTeam.back() += s[i] - '0';
				if (firstTeam.back() > 2 * N) {
					commentsfile << "player ID out of range (ID = " << firstTeam.back();
					for (unsigned j = i + 1; j < s.size() && isdigit(s[j]); j++)
						commentsfile << s[j];
					commentsfile << ", number of teams = " << 2 * N << ")." << endl;
					goto wrongAnswer;
				}
			}
			else {
				doPush = true;
			}
		}
		// done reading team output, now check it:
		if ((int) firstTeam.size() != N) {
			commentsfile << "wrong number of team members (N = " << N << ", but found " << firstTeam.size() << " team members)." << endl;
			goto wrongAnswer;
		}
		for (int i = 0; i < N; i++) {
			if (firstTeam[i] < 1 || firstTeam[i] > 2 * N) {
				commentsfile << "player ID out of range (ID = " << firstTeam[i] << "; the allowed range is 1 through " << 2 * N << ")." << endl;
				goto wrongAnswer;
			}
			if (i > 0 && firstTeam[i] <= firstTeam[i - 1]) {
				commentsfile << "team members are not sorted in ascending order." << endl;
				goto wrongAnswer;
			}
		}
		for (int i = 0; i < MAX_TEAMS; i++) teamNum[i] = 2;
		for (int i = 0; i < N; i++) teamNum[firstTeam[i] - 1] = 1;
		for (int i = 0; i < 2 * N; i++) {
			if (teamNum[i] != 1) continue;
			for (int j = 0; j < 2 * N; j++) {
				if (teamNum[j] != 2) continue;
				if (matchesPlayed[i][j] == 0) {
					commentsfile << "players " << i + 1 << " and " << j + 1 << " didn't play a match, so they should be on the same team." << endl;
					goto wrongAnswer;
				}
			}
		}
		commentsfile << "OK";
		if (whitespacefail) {
			whitespacecount++;
			commentsfile << " (but with malformed whitespace -- I'm configured to ignore this).";
		}
		else {
			commentsfile << "!";
		}
		commentsfile << endl;
		continue;
		
		wrongAnswer: numWrongAnswers++;
	}
	if (cin.fail() || cin.eof()) {
		#ifdef IGNORE_FINAL_NEWLINE
			commentsfile << "Warning: missing final newline (I'm configured to allow this)." << endl;
		#else
			commentsfile << "Problem with team output: missing final newline." << endl;
			numWrongAnswers++;
		#endif
	}
	if (getline(cin, s)) {
		#ifdef IGNORE_ADDITIONAL_DATA
			commentsfile << "Warning: additional data after final test case (I'm configured to allow this)." << endl;
		#else
			commentsfile << "Problem with team output: additional data after final test case." << endl;
			numWrongAnswers++;
		#endif
	}
	commentsfile << "\nSummary: ";
	if (numWrongAnswers > 0) {
		if (numWrongAnswers == 1) {
			commentsfile << "there was 1 problem." << endl;
		}
		else {
			commentsfile << "there were " << numWrongAnswers << " problems." << endl;
		}
		return 43;
	}
	else {
		commentsfile << "all OK";
		if (whitespacecount > 0) {
			if (whitespacecount > 0) {
				if (whitespacecount == 1) {
					commentsfile << " (1 whitespace-related warning).";
				}
				else {
					commentsfile << " (" << whitespacecount << " whitespace-related warnings).";
				}
			}
		}
		else {
			commentsfile << "!";
		}
		commentsfile << endl;
		return 42;
	}
}
