// Checker for problem `Pacman', written for DOMjudge 5.0.0.
// For all test cases:
// * read in test case
// * compute solution (with algorithm from pacmanRudy.cc)
//   (alternative would be to read in the correct output from a separate file)
// * read in (and parse) output from submission
// * verify output
//
// Usage:
// * compile with g++:  g++ -Wall pacmanchecker.cc -o pacmanchecker
// * move executable pacmanchecker into directory with:
//   - testdata.in (or some other file containing the test input)
//   - testdata.out (or some other file containing the test output).
//     In fact, we do NOT use this file, but we need to include its name
//     in the call of this checker
//   - a file test.out (or some other file containing the output to be checked)
// * ./pacmanchecker testdata.in testdata.out feedbackdir < test.out
//   or
//   ./pacmanchecker testdata.in testdata.out feedbackdir --verbose < test.out
// * here, feedbackdir is a directory where feedback from the checker is
//   written into a file "judgemessage.txt"
//
// Rudy van Vliet, 16 September, 2015


#include<iostream>
#include<cstring>
#include<fstream>

using namespace std;
const int MaxFileNameLength = 1024;
ofstream fout;
const int MaxLineLength = 2500;
bool verbose = false;

enum Direction {South, East, North, West};
const int MaxDim = 50;
char grid[MaxDim][MaxDim+1];  // +1 to store EOS at read-in
bool endNode[MaxDim*MaxDim*MaxDim*MaxDim];
int queue[MaxDim*MaxDim*MaxDim*MaxDim],
    mydistance[MaxDim*MaxDim*MaxDim*MaxDim],
    parent[MaxDim*MaxDim*MaxDim*MaxDim];
Direction direction[MaxDim*MaxDim*MaxDim*MaxDim],
          path[MaxDim*MaxDim*MaxDim*MaxDim];
const char Pacman = 'P';
const char Wall = 'X';
const char Ghost = 'G';

//*************************************************************************

bool IntegerInRange (const char *variable, int val, int minVal, int maxVal)
{
  if (val>=minVal && val<=maxVal)
    return true;
  else
  { fout << variable << "=" << val << ", but should be in [" << minVal
         << "," << maxVal << "]" << endl;
    return false;
  }

}  // IntegerInRange

//*************************************************************************

int determineNumber (int h, int w, int i1, int j1, int i2, int j2)
{ return (i1*w*h*w + j1*h*w + i2*w + j2);
}

void decodeNumber (int nr, int h, int w, int &i1, int &j1, int &i2, int &j2)
{ 
  j2 = nr%w;
  nr = nr/w;
  i2 = nr%h;
  nr = nr/h;
  j1 = nr%w;
  nr = nr/w;
  i1 = nr;

}  // decodeNumber

//*************************************************************************

  // determine position south of (i1,j1)
void neighbourSouth (int h, int i1, int j1, int &i3, int &j3)
{
  i3 = (i1+1)%h;
  j3 = j1;
  if (grid[i3][j3]==Wall)
    i3 = i1;
}

  // determine position east of (i1,j1)
void neighbourEast (int w, int i1, int j1, int &i3, int &j3)
{
  i3 = i1;
  j3 = (j1+1)%w;
  if (grid[i3][j3]==Wall)
    j3 = j1;
}

  // determine position north of (i1,j1)
void neighbourNorth (int h, int i1, int j1, int &i3, int &j3)
{
  i3 = (i1-1+h)%h;
  j3 = j1;
  if (grid[i3][j3]==Wall)
    i3 = i1;
}

  // determine position west of (i1,j1)
void neighbourWest (int w, int i1, int j1, int &i3, int &j3)
{
  i3 = i1;
  j3 = (j1-1+w)%w;
  if (grid[i3][j3]==Wall)
    j3 = j1;
}

//*************************************************************************

void initGraph (int h, int w)
{ int size,
      nr,
      i, j;

  size = h*w*h*w;

  for (nr=0;nr<size;nr++)
    endNode[nr] = false;

  for (i=0;i<h;i++)
    for (j=0;j<w;j++)
    { nr = determineNumber (h, w, i, j, i, j);
      endNode[nr] = true;
    }
      
}

//*************************************************************************

// Test if a neighbour nr1 of the current node nr0 in the BFS
// * is visited for the first time
// * and if so, if it is an endNode
// If the first condition is met, and the second is not,
// then add nr1 to the queue.
//
// Return true: if both conditions are met
//        false: otherwise
bool testNeighbourBFS (int nr0, int nr1, Direction direct, int newDist,
                       int &last)
{ bool found;

  found = false;

  if (newDist<mydistance[nr1])  // not reached before
  { mydistance[nr1] = newDist;
    parent[nr1] = nr0;
    direction[nr1] = direct;
    if (endNode[nr1])
      found = true;
    else
    { last ++;
      queue[last] = nr1;
    }
  }

  return found;

}  // testNeighbourBFS

//*************************************************************************

int BFS (int h, int w, int nr0)
{ int size,
      newDist,
      first, last,
      nr, nr1,
      i1, j1, i2, j2,
      i3, j3, i4, j4;
  bool found;

  size = h*w*h*w;

  if (endNode[nr0])
    return 0;
  else
  {
    first = 0;
    last = -1;

    for (nr=0;nr<size;nr++)
      mydistance[nr] = size+1;

    mydistance[nr0] = 0;
    parent[nr0] = -1;

    last ++;
    queue[last] = nr0;
    found = false;
    while (!found && last>=first)  // queue not empty
    { nr0 = queue[first];
      first ++;
      newDist = mydistance[nr0]+1;

      decodeNumber (nr0, h, w, i1, j1, i2, j2);

      neighbourSouth (h, i1, j1, i3, j3);
      neighbourSouth (h, i2, j2, i4, j4);
      if (grid[i3][j3]!=Ghost && grid[i4][j4]!=Ghost)
      { nr1 = determineNumber (h, w, i3, j3, i4, j4);
        if (nr0!=nr1)
        {
          found = testNeighbourBFS (nr0, nr1, South, newDist, last);
        }
      }

      if (!found)
      { neighbourEast (w, i1, j1, i3, j3);
        neighbourEast (w, i2, j2, i4, j4);
        if (grid[i3][j3]!=Ghost && grid[i4][j4]!=Ghost)
        { nr1 = determineNumber (h, w, i3, j3, i4, j4);
          if (nr0!=nr1)
          {
            found = testNeighbourBFS (nr0, nr1, East, newDist, last);
          }
        }
      }  // !found

      if (!found)
      {
        neighbourNorth (h, i1, j1, i3, j3);
        neighbourNorth (h, i2, j2, i4, j4);
        if (grid[i3][j3]!=Ghost && grid[i4][j4]!=Ghost)
        { nr1 = determineNumber (h, w, i3, j3, i4, j4);
          if (nr0!=nr1)
          {
            found = testNeighbourBFS (nr0, nr1, North, newDist, last);
          }
        }
      }  // !found

      if (!found)
      {
        neighbourWest (w, i1, j1, i3, j3);
        neighbourWest (w, i2, j2, i4, j4);
        if (grid[i3][j3]!=Ghost && grid[i4][j4]!=Ghost)
        { nr1 = determineNumber (h, w, i3, j3, i4, j4);
          if (nr0!=nr1)
          {
            found = testNeighbourBFS (nr0, nr1, West, newDist, last);
          }
        }
      }  // !found


    }  // while queue not empty

    if (found)  // path found, nr1 is endnode
      return mydistance[nr1];
    else
      return -1;
  
  }  // else

}  // BFS

//*************************************************************************

// Is character c a digit?
bool isDigit (char c)
{
  if (c>='0' && c<='9')
    return true;
  else
    return false;

}  // isDigit

//*************************************************************************

// Is character c a valid direction?
bool isDirection (char c)
{
  if (c=='S' || c=='E' || c=='N' || c=='W')
    return true;
  else
    return false;

}  // isDirection

//*************************************************************************

// Split lineOutput into distanceOutut and pathOutput.
// Return true, if lineOutput has correct format
//        false, otherwise
bool parseOutput (char *lineOutput, int &distanceOutput, char *pathOutput)
{ int len,
      i, j,
      leadingZeroes;

  len = strlen (lineOutput);
  i = 0;

  distanceOutput = 0;
  leadingZeroes = 0;
  while (i<len && isDigit(lineOutput[i]))
  { distanceOutput = 10*distanceOutput + (lineOutput[i] - '0');
    if (distanceOutput==0)
      leadingZeroes ++;
    i++;
  }

  if (i==0)  // lineOutput does not start with a (nonnegative) number
  { if (verbose)
      fout << "  Line does not start with digit" << endl;
    return false;
  }

  if (distanceOutput!=0 && leadingZeroes>0)
  { if (verbose)
      fout << "  Line has leading zeroes" << endl;
    return false;
  }

  if (distanceOutput==0 && leadingZeroes>1)
  { if (verbose)
      fout << "  Line has leading zeroes" << endl;
    return false;
  }

  if (i==len)
  { if (verbose)
      fout << "  Line only contains number" << endl;
    return false;
  }

  if (lineOutput[i] != ' ')
  { if (verbose)
      fout << "  Number is followed by non-space" << endl;
    return false;
  }

  i++;
  j=0;
  while (i<len)
  {
    if (isDirection (lineOutput[i]))
    { pathOutput[j] = lineOutput[i];
      i++;
      j++;
    }
    else
    { if (verbose)
        fout << "  Character " << i+1 << " is not a valid direction" << endl;
      return false;
    }
    
  }

  pathOutput[j] = '\0';
  return true;

}  // parseOutput

//*************************************************************************

bool checkPath (int h, int w, int nr0, char *pathOutput)
{ int len, i,
      i1, j1, i2, j2,
      i3, j3, i4, j4,
      nr1;
  bool OK,
       found;  // found an endnode
 
  len = strlen (pathOutput);
  OK = true;
  found = false;

  for (i=0;OK && !found && i<len;i++)
  { decodeNumber (nr0, h, w, i1, j1, i2, j2);

    switch (pathOutput[i])
    { case 'S':
        neighbourSouth (h, i1, j1, i3, j3);
        neighbourSouth (h, i2, j2, i4, j4);
        break;
      case 'E':
        neighbourEast (w, i1, j1, i3, j3);
        neighbourEast (w, i2, j2, i4, j4);
        break;
      case 'N':
        neighbourNorth (h, i1, j1, i3, j3);
        neighbourNorth (h, i2, j2, i4, j4);
        break;
      case 'W':
        neighbourWest (w, i1, j1, i3, j3);
        neighbourWest (w, i2, j2, i4, j4);
        break;
      default:  // just to avoid compiler warnings...
        i3 = i1;
        j3 = j1;
        i4 = i2;
        j4 = j2;
    }

    if (grid[i3][j3]!=Ghost && grid[i4][j4]!=Ghost)
    { nr1 = determineNumber (h, w, i3, j3, i4, j4);
      if (nr0!=nr1)
      {
        if (endNode[nr1])
          found = true;
        else
          nr0 = nr1;  // continue from new node
      }
      else  // nr0==nr1, hence no real move
      { OK = false;
        if (verbose)
          fout << "  Move " << i+1 << " is no real move" << endl;
      }
    }
    else  // bumped into a ghost
    { OK = false;
      if (verbose)
        fout << "  Move " << i+1 << " bumps into a ghost" << endl;
    }
  }  // for i

  if (i<len && found)  // this is not possible, if jury's solution is correct
  { if (verbose)
      fout << "  Path leads to endnode too early" << endl;
  }

  if (i==len && OK && found)
    return true;
  else
    return false;

}  // checkPath

//*************************************************************************

int main(int argc, char **argv)
{ int nInst, inst;
  int h, w,
      i, j,
      i1, j1, i2, j2,
      nr0;
  int Pfound;
  ifstream fin;
  int  result,
       distanceOutput;
  char lineOutput[MaxLineLength+1],
       pathOutput[MaxLineLength+1];
  bool OK,
       allOK;
  char judgeMessageFile[MaxFileNameLength+1];

  if (argc<4)
  { cerr << "Too few arguments for DOMjudge 5.0.0" << endl;
    cerr << "Usage:" << endl;
    cerr << "  pacmanchecker <testdata.in> <testdata.ans> <feedbackdir> < <program.out>" << endl;
    cerr << "or" << endl;
    cerr << "  pacmanchecker <testdata.in> <testdata.ans> <feedbackdir> --verbose < <program.out>" << endl;
    return 1;
  }

  if (argc>4 && strcmp(argv[4],"--verbose")==0)
    verbose = true;

  fin.open (argv[1], ios::in);
  if (!fin.is_open())
  { cerr << "File '" << argv[1]
         << "' could not be opened for reading." << endl;
    return 1;
  }

    // we do not use argv[2], the jury reference output,
    // but compute the answer ourselves

  strcpy (judgeMessageFile, argv[3]);
  strcat (judgeMessageFile, "/judgemessage.txt");
  fout.open (judgeMessageFile, ios::out);
  if (!fout.is_open())
  { cerr << "File '" << judgeMessageFile
         << "' could not be opened for writing." << endl;
    fin.close();
    return 1;
  }

  allOK = true;

  fin >> nInst;
  for (inst=1;inst<=nInst;inst++)
  {
    fout << inst << ": ";

    fin >> h >> w;
    if (IntegerInRange ("h", h, 2, MaxDim)
        && IntegerInRange ("w", w, 2, MaxDim))
    {
      for (i=0;i<h;i++)
        fin >> grid[i];

        // determine position of Pacmen
      Pfound = 0;
      for (i=0;i<h;i++)
        for (j=0;j<w;j++)
          if (grid[i][j]==Pacman)
          { if (Pfound==0)
            { i1 = i;
              j1 = j;
              Pfound ++;
            }
            else
            { i2 = i;
              j2 = j;
              Pfound ++;
            }
          }

      if (Pfound==2)  // input is OK
      {
        nr0 = determineNumber (h, w, i1, j1, i2, j2);
          // start node of search

        initGraph (h, w);

        result = BFS (h, w, nr0);

        cin.getline (lineOutput, MaxLineLength+1);

        if (result==-1)
        { if (strcmp (lineOutput, "IMPOSSIBLE") != 0)
          { fout << "WRONG ANSWER";
            if (verbose)
              fout << ": POSSIBLE versus IMPOSSIBLE";
            fout << endl;
            allOK = false;
          }
          else
            fout << "OK" << endl;  // both solutions report "IMPOSSIBLE"
        }
        else  // result != -1
        { if (strcmp (lineOutput, "IMPOSSIBLE") == 0)
          { fout << "WRONG ANSWER";
            if (verbose)
              fout << ": IMPOSSIBLE versus POSSIBLE";
            fout << endl;
            allOK = false;
          }
          else  // both solutions report that there is a path
          { OK = parseOutput (lineOutput, distanceOutput, pathOutput);
            if (!OK)
            { fout << "WRONG ANSWER";
              if (verbose) // or presentation error
                fout << ": output could not be parsed";
              fout << endl;
              allOK = false;
            }
            else  // output could be parsed
            { if (distanceOutput!=result)
              { fout << "WRONG ANSWER";
                if (verbose)
                  fout << ": different path lengths";
                fout << endl;
                allOK = false;
              }
                  // different path lengths
              else
              { if ((int) strlen (pathOutput) != distanceOutput)
                { fout << "WRONG ANSWER";
                  if (verbose)
                    fout << ": inconsistent output";
                  fout << endl;  // inconsistent output
                  allOK = false;
                }
                else
                { OK = checkPath (h, w, nr0, pathOutput);
                  if (OK)
                    fout << "OK" << endl;
                  else
                  { fout << "WRONG ANSWER";
                    if (verbose)
                      fout << ": path is not correct";
                    fout << endl;
                    allOK = false;
                  }
                }
              }

            }  // output could be parsed

          }  // both solutions report that there is a path

        }
      }  // input OK
      else
      { fout << "Grid does not contain exactly two Pacmen.\n";
        allOK = false;
      }

    }  // if IntegerInRange
    else
    {   // ignore rest of input
      fin.close ();
      fout.close ();
      return 1;
    }

  }

  if (cin.getline (lineOutput, MaxLineLength+1))
        // Undocumented feature of cin: if there is anything more to read
        // from standard input, then return value is true
  { fout << "WRONG ANSWER";
    if (verbose)
      fout << ": more output than expected";
    fout << endl;
    allOK = false;
  }

  fin.close ();
  fout.close ();

  if (allOK)
    return 42;  // `Accepted' according to DOMjudge
  else
    return 43;  // `Wrong Answer' according to DOMjudge

}
