#include <algorithm>
#include <cmath>
#include <fstream>
#include <iostream>
#include <vector>

using namespace std;

struct Vector
{
	Vector() : x(0.0), y(0.0), z(0.0) {}
	Vector(double x, double y, double z) : x(x), y(y), z(z) {}
	double x;
	double y;
	double z;
	double norm()
	{
		return sqrt(x*x+y*y+z*z);
	}
	Vector normed();
};

Vector operator+(const Vector& lhs, const Vector& rhs)
{
	return Vector(lhs.x + rhs.x, lhs.y + rhs.y, lhs.z + rhs.z);
}
Vector operator-(const Vector& lhs, const Vector& rhs)
{
	return Vector(lhs.x - rhs.x, lhs.y - rhs.y, lhs.z - rhs.z);
}
Vector operator*(const Vector& lhs, double rhs)
{
	return Vector(lhs.x * rhs, lhs.y * rhs, lhs.z * rhs);
}
Vector operator*(double lhs, const Vector& rhs)
{
	return Vector(lhs * rhs.x, lhs * rhs.y, lhs * rhs.z);
}
Vector operator/(const Vector& lhs, double rhs)
{
	return Vector(lhs.x / rhs, lhs.y / rhs, lhs.z / rhs);
}
Vector Vector::normed()
{
	return (*this) * (1/this->norm());
}

double dotProduct(Vector a, Vector b)
{
	return a.x * b.x + a.y * b.y + a.z * b.z;
}

Vector crossProduct(Vector u, Vector v)
{
	return Vector(u.y * v.z - u.z * v.y,
                  u.z * v.x - u.x * v.z,
                  u.x * v.y - u.y * v.x);
}

struct Hit
{
	double t;
	Vector new_origin;
	Vector new_direction;
	int component_hit;
	int speed_decrease;
};

struct HitComparer {
  bool operator() (const Hit& i,const Hit& j) { return i.t<j.t; }
};

struct Face
{
	vector<Vector> vectors;
	Vector first() { return vectors[0]; }
	Vector second() { return vectors[1]; }
	Vector third() { return vectors[2]; }
	Vector direction1() { return first() - second(); }
	Vector direction2() { return first() - third(); }
	Vector normal() { return crossProduct(direction1(), direction2()); }
	Vector normed_normal() { return normal().normed(); }
	int component;
	int speed_decrease;
	int deflection_threshold;
};

int f, c;
int D, s;
Vector origin;
Vector direction;

vector<Face> faces;

bool intersects(Face current_face, Vector current_direction)
{
	return abs(dotProduct(current_direction, current_face.normal())) > 1e-9;
}

double intersection_point(Face current_face, Vector current_origin, Vector current_direction)
{
	return dotProduct((current_face.first() - current_origin), current_face.normal()) / dotProduct(current_direction, current_face.normal());
}

Vector get_point(double t, Vector current_origin, Vector current_direction)
{
	return current_origin + current_direction * t;
}

struct Vector2d
{
	Vector2d() : x(0.0), y(0.0) {}
	Vector2d(double x, double y) : x(x), y(y) {}
	double x;
	double y;
};

Vector2d operator-(const Vector2d& lhs, const Vector2d& rhs)
{
	return Vector2d(lhs.x - rhs.x, lhs.y - rhs.y);
}

double crossProduct2d(Vector2d u, Vector2d v)
{
	return u.x * v.y - u.y * v.x;
}

Vector project(Vector v, Vector u)
{
	return dotProduct(v, u) / dotProduct(u, u) * u;
}

bool within_face(Face current_face, Vector intersection_point)
{
	vector<Vector2d> rotated_vectors;
	Vector2d rotated_intersection_point;
	{
		Vector P = current_face.first();
		Vector Q = current_face.second();
		Vector R = current_face.third();
		Vector S_prime = Q - P;
		Vector S = S_prime / S_prime.norm();
		Vector T_prime = R - P;
		Vector T_prime2 = T_prime - project(T_prime, S_prime);
		Vector T = T_prime2 / T_prime2.norm();
		for (int i = 0; i < current_face.vectors.size(); i++)
		{
			Vector2d mapped(dotProduct(S, current_face.vectors[i] - P), dotProduct(T, current_face.vectors[i] - P));
			rotated_vectors.push_back(mapped);
		}
		rotated_intersection_point = Vector2d(dotProduct(S, intersection_point - P), dotProduct(T, intersection_point - P));
	}
	
	double initialCrossProduct = crossProduct2d(rotated_vectors[1]-rotated_vectors[0], rotated_intersection_point - rotated_vectors[0]);
	bool pos = initialCrossProduct > 0;
	for (int i = 1; i < rotated_vectors.size(); i++)
	{
		double newCrossProduct = crossProduct2d(rotated_vectors[(i+1)% rotated_vectors.size()]-rotated_vectors[i], rotated_intersection_point - rotated_vectors[i]);
		bool new_pos = newCrossProduct > 0;
		if (new_pos != pos)
		{
			return false;
		}
	}
	return true;
}

Vector deflect(Vector current_direction, Face current_face)
{
	return current_direction - 2 * dotProduct(current_direction, current_face.normed_normal()) * current_face.normed_normal();
}

vector<Hit> find_hits(Vector current_origin, Vector current_direction, int current_speed)
{
	vector<Hit> result;
	
	for (unsigned int i = 0; i < faces.size(); i++)
	{
		Face current_face = faces[i];
		
		if (intersects(current_face, current_direction))
		{
			double intersection_t = intersection_point(current_face, current_origin, current_direction);
			if (intersection_t > 1e-9)
			{
				Vector intersection_point = get_point(intersection_t, current_origin, current_direction);
				if (within_face(current_face, intersection_point))
				{
					Hit new_hit;
					new_hit.t = intersection_t;
					new_hit.new_origin = intersection_point;
					new_hit.speed_decrease = current_face.speed_decrease;
					int new_speed = current_speed - current_face.speed_decrease;
					if (new_speed <= current_face.deflection_threshold)
					{
						new_hit.new_direction = deflect(current_direction, current_face);
						new_hit.component_hit = -1;
					}
					else
					{
						new_hit.new_direction = current_direction;
						new_hit.component_hit = current_face.component;
					}
					result.push_back(new_hit);
				}
			}
		}
	}
	
	return result;
}

vector<bool> components_hit;

void read_input()
{
	cin >> f >> c;
	cin >> D >> s;
	cin >> origin.x >> origin.y >> origin.z >> direction.x >> direction.y >> direction.z;
	faces.clear();
	for (int i = 0; i < f; i++)
	{
		int v;
		Face new_face; new_face.component = -1;
		cin >> v >> new_face.speed_decrease >> new_face.deflection_threshold;
		
		for (int j = 0; j < v; j++)
		{
			Vector next_vector;
			cin >> next_vector.x >> next_vector.y >> next_vector.z;
			new_face.vectors.push_back(next_vector);
		}
		
		faces.push_back(new_face);
	}
	
	components_hit.clear();
	for (int i = 0; i < c; i++)
	{
		components_hit.push_back(false);
		Vector center, size;
		Face new_face; new_face.component = i;
		cin >> center.x >> center.y >> center.z >> size.x >> size.y >> size.z >> new_face.speed_decrease >> new_face.deflection_threshold;
		
		double minX = center.x - size.x/2.0;
		double maxX = center.x + size.x/2.0;
		double minY = center.y - size.y/2.0;
		double maxY = center.y + size.y/2.0;
		double minZ = center.z - size.z/2.0;
		double maxZ = center.z + size.z/2.0;
		new_face.vectors.push_back(Vector(minX, minY, minZ));
		new_face.vectors.push_back(Vector(minX, minY, maxZ));
		new_face.vectors.push_back(Vector(minX, maxY, maxZ));
		new_face.vectors.push_back(Vector(minX, maxY, minZ));
		faces.push_back(new_face);
		new_face.vectors.clear();
		new_face.vectors.push_back(Vector(maxX, minY, minZ));
		new_face.vectors.push_back(Vector(maxX, minY, maxZ));
		new_face.vectors.push_back(Vector(maxX, maxY, maxZ));
		new_face.vectors.push_back(Vector(maxX, maxY, minZ));
		faces.push_back(new_face);
		new_face.vectors.clear();
		new_face.vectors.push_back(Vector(minX, minY, minZ));
		new_face.vectors.push_back(Vector(minX, minY, maxZ));
		new_face.vectors.push_back(Vector(maxX, minY, maxZ));
		new_face.vectors.push_back(Vector(maxX, minY, minZ));
		faces.push_back(new_face);
		new_face.vectors.clear();
		new_face.vectors.push_back(Vector(minX, maxY, minZ));
		new_face.vectors.push_back(Vector(minX, maxY, maxZ));
		new_face.vectors.push_back(Vector(maxX, maxY, maxZ));
		new_face.vectors.push_back(Vector(maxX, maxY, minZ));
		faces.push_back(new_face);
		new_face.vectors.clear();
		new_face.vectors.push_back(Vector(minX, minY, minZ));
		new_face.vectors.push_back(Vector(minX, maxY, minZ));
		new_face.vectors.push_back(Vector(maxX, maxY, minZ));
		new_face.vectors.push_back(Vector(maxX, minY, minZ));
		faces.push_back(new_face);
		new_face.vectors.clear();
		new_face.vectors.push_back(Vector(minX, minY, maxZ));
		new_face.vectors.push_back(Vector(minX, maxY, maxZ));
		new_face.vectors.push_back(Vector(maxX, maxY, maxZ));
		new_face.vectors.push_back(Vector(maxX, minY, maxZ));
		faces.push_back(new_face);
	}
}

void write_components()
{
	int hit_count = count(components_hit.begin(), components_hit.end(), true);
	cout << hit_count << endl;
	for (int i = 0; i < c; i++)
	{
		if (components_hit[i]) cout << i << endl;
	}
}

void test()
{
	read_input();
	
	int current_speed = s;
	Vector current_origin = origin;
	Vector current_direction = direction;
	for (int i = 0; i < D; i++)
	{
		if (current_speed <= 0)
		{
			break;
		}
		
		vector<Hit> hits = find_hits(current_origin, current_direction, current_speed);
		
		if (hits.size() == 0)
		{
			write_components();
			cout << "SHOT NEVER DISINTEGRATED" << endl;
			return;
		}
		
		Hit next_hit = *min_element(hits.begin(), hits.end(), HitComparer());
		current_origin = next_hit.new_origin;
		current_direction = next_hit.new_direction;
		if (next_hit.component_hit >= 0) components_hit[next_hit.component_hit] = true;
		current_speed -= next_hit.speed_decrease;
	}
	
	write_components();
	cout << (int)floor(current_origin.x) << " " << (int)floor(current_origin.y) << " " << (int)floor(current_origin.z) << endl;
}

int main(int argc, char** argv)
{
	int cases;
	cin >> cases;
	while (cases > 0)
	{
		cases--;
		test();
	}
		
	return 0;
}
